@@ -13,6 +13,7 @@ import Data.Maybe
 import Data.Ord
 import Config.Type
 import Config.Haskell
+import qualified Data.HashSet as Set
 import Prelude
 
 
@@ -44,21 +45,29 @@ applyHints cs = applyHintsReal $ map SettingClassify cs
 
 applyHintsReal :: [Setting] -> Hint -> [(Module_, [Comment])] -> [Idea]
 applyHintsReal settings hints_ ms = concat $
-    [ map (classify $ cls ++ mapMaybe readPragma (universeBi m)) $
-        order "" (hintModule hints settings nm m) `merge`
-        concat [order (fromNamed d) $ decHints d | d <- moduleDecls m] `merge`
-        concat [order "" $ hintComment hints settings c | c <- cs]
+    [ map (classify (cls ++ mapMaybe readPragma (universeBi m)) . removeRequiresExtensionNotes m) $
+        order [] (hintModule hints settings nm m) `merge`
+        concat [order [fromNamed d] $ decHints d | d <- moduleDecls m] `merge`
+        concat [order [] $ hintComment hints settings c | c <- cs]
     | (nm,(m,cs)) <- mns
     , let decHints = hintDecl hints settings nm m -- partially apply
-    , let order n = map (\i -> i{ideaModule=moduleName m, ideaDecl=n}) . sortBy (comparing ideaSpan)
+    , let order n = map (\i -> i{ideaModule= f $ moduleName m : ideaModule i, ideaDecl= f $ n ++ ideaDecl i}) . sortBy (comparing ideaSpan)
     , let merge = mergeBy (comparing ideaSpan)] ++
     [map (classify cls) (hintModules hints settings $ map (second fst) mns)]
     where
+        f = nubOrd . filter (/= "")
         cls = [x | SettingClassify x <- settings]
         mns = map (scopeCreate . fst &&& id) ms
         hints = (if length ms <= 1 then noModules else id) hints_
         noModules h = h{hintModules = \_ _ -> []} `mappend` mempty{hintModule = \s a b -> hintModules h s [(a,b)]}
 
+-- If the hint has said you RequiresExtension Foo, but Foo is enabled, drop the note
+removeRequiresExtensionNotes :: Module_ -> Idea -> Idea
+removeRequiresExtensionNotes m = \x -> x{ideaNote = filter keep $ ideaNote x}
+    where
+        exts = Set.fromList $ map fromNamed $ moduleExtensions m
+        keep (RequiresExtension x) = not $ x `Set.member` exts
+        keep _ = True
 
 -- | Given a list of settings (a way to classify) and a list of hints, run them over a list of modules.
 executeHints :: [Setting] -> [(Module_, [Comment])] -> [Idea]
@@ -71,10 +80,8 @@ parseModuleApply flags s file src = do
     res <- parseModuleEx (parseFlagsAddFixities [x | Infix x <- s] flags) file src
     case res of
         Right m -> return $ Right m
-        Left (ParseError sl msg ctxt) -> do
-            i <- return $ rawIdeaN Error "Parse error" (mkSrcSpan sl sl) ctxt Nothing []
-            i <- return $ classify [x | SettingClassify x <- s] i
-            return $ Left i
+        Left (ParseError sl msg ctxt) ->
+            return $ Left $ classify [x | SettingClassify x <- s] $ rawIdeaN Error "Parse error" (mkSrcSpan sl sl) ctxt Nothing []
 
 
 -- | Find which hints a list of settings implies.
@@ -90,6 +97,6 @@ classify xs i =  let s = foldl' (f i) (ideaSeverity i) xs in s `seq` i{ideaSever
     where
         -- figure out if we need to change the severity
         f :: Idea -> Severity -> Classify -> Severity
-        f i r c | classifyHint c ~= ideaHint i && classifyModule c ~= ideaModule i && classifyDecl c ~= ideaDecl i = classifySeverity c
+        f i r c | classifyHint c ~= [ideaHint i] && classifyModule c ~= ideaModule i && classifyDecl c ~= ideaDecl i = classifySeverity c
                 | otherwise = r
-        x ~= y = null x || x == y
+        x ~= y = null x || x `elem` y
