@@ -48,10 +48,64 @@ data ParseFlags = ParseFlags
     ,hseFlags :: ParseMode -- ^ How the file is parsed (defaults to all fixities in the @base@ package and most non-conflicting extensions).
     }
 
+lensFixities :: [Fixity]
+lensFixities = concat
+    -- List as provided at https://github.com/ndmitchell/hlint/issues/416
+    [infixr_ 4 ["%%@~","<%@~","%%~","<+~","<*~","<-~","<//~","<^~","<^^~","<**~"]
+    ,infix_ 4 ["%%@=","<%@=","%%=","<+=","<*=","<-=","<//=","<^=","<^^=","<**="]
+    ,infixr_ 2 ["<<~"]
+    ,infixr_ 9 ["#."]
+    ,infixl_ 8 [".#"]
+    ,infixr_ 8 ["^!","^@!"]
+    ,infixl_ 1 ["&","<&>","??"]
+    ,infixl_ 8 ["^.","^@."]
+    ,infixr_ 9 ["<.>","<.",".>"]
+    ,infixr_ 4 ["%@~",".~","+~","*~","-~","//~","^~","^^~","**~","&&~","<>~","||~","%~"]
+    ,infix_ 4 ["%@=",".=","+=","*=","-=","//=","^=","^^=","**=","&&=","<>=","||=","%="]
+    ,infixr_ 2 ["<~"]
+    ,infixr_ 2 ["`zoom`","`magnify`"]
+    ,infixl_ 8 ["^..","^?","^?!","^@..","^@?","^@?!"]
+    ,infixl_ 8 ["^#"]
+    ,infixr_ 4 ["<#~","#~","#%~","<#%~","#%%~"]
+    ,infix_ 4 ["<#=","#=","#%=","<#%=","#%%="]
+    ,infixl_ 9 [":>"]
+    ,infixr_ 4 ["</>~","<</>~","<.>~","<<.>~"]
+    ,infix_ 4 ["</>=","<</>=","<.>=","<<.>="]
+    ,infixr_ 4 [".|.~",".&.~","<.|.~","<.&.~"]
+    ,infix_ 4 [".|.=",".&.=","<.|.=","<.&.="]
+    ]
+
+hspecFixities :: [Fixity]
+hspecFixities =
+    infix_ 1 ["`shouldBe`","`shouldSatisfy`","`shouldStartWith`","`shouldEndWith`","`shouldContain`","`shouldMatchList`"
+             ,"`shouldReturn`","`shouldNotBe`","`shouldNotSatisfy`","`shouldNotContain`","`shouldNotReturn`","`shouldThrow`"]
+
+
+-- Fixites from the `base` package which are currently
+-- missing from `haskell-src-exts`'s baseFixities.
+-- see https://github.com/haskell-suite/haskell-src-exts/pull/400
+baseNotYetInHSE :: [Fixity]
+baseNotYetInHSE = concat
+    [infixr_ 9 ["`Compose`"]
+    ,infixr_ 6 ["<>"]
+    ,infixr_ 5 ["<|"]
+    ,infixl_ 4 ["<$!>","<$","$>"]
+    ,infix_ 4 [":~:", ":~~:"]
+    ]
+
+customFixities :: [Fixity]
+customFixities =
+    infixl_ 1 ["`on`"]
+        -- see https://github.com/ndmitchell/hlint/issues/425
+        -- otherwise GTK apps using `on` at a different fixity have spurious warnings
+
 -- | Default value for 'ParseFlags'.
 defaultParseFlags :: ParseFlags
-defaultParseFlags = ParseFlags NoCpp
-    defaultParseMode{fixities=Just baseFixities, ignoreLinePragmas=False, ignoreFunctionArity=True, extensions=defaultExtensions}
+defaultParseFlags = ParseFlags NoCpp defaultParseMode
+    {fixities = Just $ customFixities ++ baseFixities ++ baseNotYetInHSE ++ lensFixities ++ hspecFixities
+    ,ignoreLinePragmas = False
+    ,ignoreFunctionArity = True
+    ,extensions = defaultExtensions}
 
 parseFlagsNoLocations :: ParseFlags -> ParseFlags
 parseFlagsNoLocations x = x{cppFlags = case cppFlags x of Cpphs y -> Cpphs $ f y; y -> y}
