@@ -5,7 +5,8 @@ module HSE.Util(module HSE.Util, def) where
 import Control.Monad
 import Data.Default
 import Data.List
-import Language.Haskell.Exts.Util() -- Orphan instances of Default for SrcLoc etc
+import Language.Haskell.Exts.Util
+import Control.Monad.Trans.State
 import Data.Maybe
 import Data.Data hiding (Fixity)
 import System.FilePath
@@ -46,6 +47,9 @@ modulePragmas :: Module_ -> [ModulePragma S]
 modulePragmas (Module _ _ x _ _) = x
 modulePragmas _ = [] -- XmlPage/XmlHybrid
 
+moduleExtensions :: Module_ -> [Name S]
+moduleExtensions x = concat [y | LanguagePragma _ y <- modulePragmas x]
+
 fromModuleName :: ModuleName S -> String
 fromModuleName (ModuleName _ x) = x
 
@@ -82,7 +86,7 @@ fromTyBang (TyBang _ _ _ x) = x
 fromTyBang x = x
 
 fromDeriving :: Deriving s -> [InstRule s]
-fromDeriving (Deriving _ x) = x
+fromDeriving (Deriving _ _ x) = x
 
 -- is* :: Exp_ -> Bool
 -- is* :: Decl_ -> Bool
@@ -167,6 +171,10 @@ dotApps [] = error "HSE.Util.dotApps, does not work on an empty list"
 dotApps [x] = x
 dotApps (x:xs) = dotApp x (dotApps xs)
 
+isReturn :: Exp_ -> Bool
+-- Allow both pure and return, as they have the same semantics
+isReturn (Var _ (UnQual _ (Ident _ x))) = x == "return" || x == "pure"
+isReturn _ = False
 
 isLexeme Var{} = True
 isLexeme Con{} = True
@@ -189,6 +197,37 @@ isWHNF (App _ c@Con{} _) | prettyPrint c `elem` ["Just","Left","Right"] = True
 isWHNF _ = False
 
 
+-- | Like needBracket, but with a special case for a . b . b, which
+--   was removed from haskell-src-exts-util-0.2.2
+needBracketOld :: Int -> Exp_ -> Exp_ -> Bool
+needBracketOld i parent child
+    | isDotApp parent, isDotApp child, i == 1 = False
+    | otherwise = needBracket i parent child
+
+transformBracketOld :: (Exp_ -> Maybe Exp_) -> Exp_ -> Exp_
+transformBracketOld op = snd . g
+    where
+        g = f . descendBracketOld g
+        f x = maybe (False,x) ((,) True) (op x)
+
+-- | Descend, and if something changes then add/remove brackets appropriately
+descendBracketOld :: (Exp_ -> (Bool, Exp_)) -> Exp_ -> Exp_
+descendBracketOld op x = descendIndex g x
+    where
+        g i y = if a then f i b else b
+            where (a,b) = op y
+
+        f i (Paren _ y) | not $ needBracketOld i x y = y
+        f i y           | needBracketOld i x y = addParen y
+        f _ y           = y
+
+descendIndex :: Data a => (Int -> a -> a) -> a -> a
+descendIndex f x = flip evalState 0 $ flip descendM x $ \y -> do
+    i <- get
+    modify (+1)
+    return $ f i y
+
+
 ---------------------------------------------------------------------
 -- HSE FUNCTIONS
 
