@@ -54,12 +54,18 @@ foo = (case x of y -> z; q -> w) :: Int
 -- backup fixity resolution
 main = do a += b . c; return $ a . b
 
+-- <$> bracket tests
+yes = (foo . bar x) <$> baz q -- foo . bar x <$> baz q
+no = foo . bar x <$> baz q
+
 -- annotations
 main = 1; {-# ANN module ("HLint: ignore Use camelCase" :: String) #-}
 main = 1; {-# ANN module (1 + (2)) #-} -- 2
 
--- special cases (from esqueleto, see #224)
+-- special case from esqueleto, see #224
 main = operate <$> (select $ from $ \user -> return $ user ^. UserEmail)
+-- unknown fixity, see #426
+bad x = x . (x +? x . x)
 </TEST>
 -}
 
@@ -137,15 +143,18 @@ dollar = concatMap f . universe
     where
         f x = [suggest "Redundant $" x y [r] | InfixApp _ a d b <- [x], opExp d ~= "$"
               ,let y = App an a b, not $ needBracket 0 y a, not $ needBracket 1 y b
-              ,let r = Replace Expr (toSS x) [("a", toSS a), ("b", toSS b)] "a b"
-                ]
-
+              ,let r = Replace Expr (toSS x) [("a", toSS a), ("b", toSS b)] "a b"]
               ++
               [suggest "Move brackets to avoid $" x (t y) [r] |(t, e@(Paren _ (InfixApp _ a1 op1 a2))) <- splitInfix x
               ,opExp op1 ~= "$", isVar a1 || isApp a1 || isParen a1, not $ isAtom a2
               ,not $ a1 ~= "select" -- special case for esqueleto, see #224
               , let y = App an a1 (Paren an a2)
               , let r = Replace Expr (toSS e) [("a", toSS a1), ("b", toSS a2)] "a (b)" ]
+              ++
+              -- special case of (v1 . v2) <$> v3
+              [suggest "Redundant bracket" x y []
+              | InfixApp _ (Paren _ o1@(InfixApp _ v1 (isDot -> True) v2)) o2 v3 <- [x], opExp o2 ~= "<$>"
+              , let y = InfixApp an o1 o2 v3]
 
 
 -- return both sides, and a way to put them together again
