@@ -31,7 +31,10 @@ data Set (cxt :: * -> *) a = Set [a]
 {-# LANGUAGE RecordWildCards #-} \
 record field = Record{..}
 {-# LANGUAGE RecordWildCards #-} \
-record = 1 --
+record = 1 -- @Note may require `{-# LANGUAGE DisambiguateRecordFields #-}` adding to the top of the file
+{-# LANGUAGE RecordWildCards #-} \
+{-# LANGUAGE DisambiguateRecordFields #-} \
+record = 1 -- @NoNote
 {-# LANGUAGE UnboxedTuples #-} \
 record = 1 --
 {-# LANGUAGE TemplateHaskell #-} \
@@ -123,7 +126,7 @@ import Data.List.Extra
 import Data.Ratio
 import Data.Data
 import Refact.Types
-import Data.Monoid
+import Data.Semigroup
 import Prelude
 
 
@@ -146,7 +149,7 @@ minimalExtensions x es = nub $ concatMap f es
 
 
 -- RecordWildCards implies DisambiguateRecordFields, but most people probably don't want it
-warnings old new | wildcards `elem` old && wildcards `notElem` new = [Note "you may need to add DisambiguateRecordFields"]
+warnings old new | wildcards `elem` old && wildcards `notElem` new = [RequiresExtension "DisambiguateRecordFields"]
     where wildcards = EnableExtension RecordWildCards
 warnings _ _ = []
 
@@ -244,10 +247,12 @@ data Derives = Derives
     ,derivesData :: [String]
     ,derivesStandalone :: [String]
     }
+instance Semigroup Derives where
+    Derives x1 x2 x3 <> Derives y1 y2 y3 =
+        Derives (x1++y1) (x2++y2) (x3++y3)
 instance Monoid Derives where
     mempty = Derives [] [] []
-    mappend (Derives x1 x2 x3) (Derives y1 y2 y3) =
-        Derives (x1++y1) (x2++y2) (x3++y3)
+    mappend = (<>)
 
 -- | What is derived on newtype, and on data type
 --   'deriving' declarations may be on either, so we approximate as both newtype and data
@@ -264,11 +269,11 @@ derives m = mconcat $ map decl (childrenBi m) ++ map idecl (childrenBi m)
         decl (GDataDecl _ dn _ _ _ _ ds) = g dn ds
         decl (DataInsDecl _ dn _ _ ds) = g dn ds
         decl (GDataInsDecl _ dn _ _ _ ds) = g dn ds
-        decl (DerivDecl _ _ hd) = mempty{derivesStandalone=[ir hd]}
+        decl (DerivDecl _ _ _ hd) = mempty{derivesStandalone=[ir hd]}
         decl _ = mempty
 
         g dn ds = if isNewType dn then mempty{derivesNewType=xs} else mempty{derivesData=xs}
-            where xs = maybe [] (map ir . fromDeriving) ds
+            where xs = concatMap (map ir . fromDeriving) ds
 
         ir (IRule _ _ _ x) = ih x
         ir (IParen _ x) = ir x
