@@ -20,6 +20,13 @@ yes = do bar; a <- foo; return a -- do bar; foo
 no = do bar; a <- foo; return b
 yes = do x <- bar; x -- do join bar
 no = do x <- bar; x; x
+yes = do x <- bar; return (f x) -- do f <$> bar
+yes = do x <- bar; return $ f x -- do f <$> bar
+yes = do x <- bar; pure $ f x -- do f <$> bar
+yes = do x <- bar; return $ f (g x) -- do f . g <$> bar
+yes = do x <- bar; return (f $ g x) -- do f . g <$> bar
+yes = do x <- bar $ baz; return (f $ g x)
+no = do x <- bar; return (f x x)
 {-# LANGUAGE RecursiveDo #-}; no = mdo hook <- mkTrigger pat (act >> rmHook hook) ; return hook
 yes = do x <- return y; foo x -- @Suggestion do let x = y; foo x
 yes = do x <- return $ y + z; foo x -- do let x = y + z; foo x
@@ -51,7 +58,7 @@ import qualified Refact.Types as R
 import Prelude
 
 
-badFuncs = ["mapM","foldM","forM","replicateM","sequence","zipWithM"]
+badFuncs = ["mapM","foldM","forM","replicateM","sequence","zipWithM","traverse","for","sequenceA"]
 
 
 monadHint :: DeclHint
@@ -63,6 +70,7 @@ monadExp decl (parent, x) = case x of
         (view -> App2 op x1 (view -> LamConst1 _)) | op ~= ">>=" -> f x1
         Do _ xs -> [warn "Redundant return" x (Do an y) rs | Just (y, rs) <- [monadReturn xs]] ++
                    [warn "Use join" x (Do an y) rs | Just (y, rs) <- [monadJoin xs ['a'..'z']]] ++
+                   [warn "Use <$>" x (Do an y) rs | Just (y, rs) <- [monadFmap xs]] ++
                    [warn "Redundant do" x y [Replace Expr (toSS x) [("y", toSS y)] "y"]
                         | [Qualifier _ y] <- [xs], not $ doOperator parent y] ++
                    [suggest "Use let" x (Do an y) rs | Just (y, rs) <- [monadLet xs]] ++
@@ -95,9 +103,22 @@ monadCall (replaceBranches -> (bs@(_:_), gen)) | all isJust res
 monadCall x | x2:_ <- filter (x ~=) badFuncs = let x3 = x2 ++ "_" in  Just (x3, toNamed x3, [Replace Expr (toSS x) [] x3])
 monadCall _ = Nothing
 
+monadFmap :: [Stmt S] -> Maybe ([Stmt S], [Refactoring R.SrcSpan])
+monadFmap (reverse -> q@(Qualifier _ (let go (App _ f x) = first (f:) $ go (fromParen x)
+                                          go (InfixApp _ f (isDol -> True) x) = first (f:) $ go x
+                                          go x = ([], x)
+                                      in go -> (ret:f:fs, view -> Var_ v))):g@(Generator _ (view -> PVar_ u) x):rest)
+    | isReturn ret, notDol x, u == v, null rest, v `notElem` vars (f:fs)
+    = Just (reverse (Qualifier an (InfixApp an (foldl' (flip (InfixApp an) (toNamed ".")) f fs) (toNamed "<$>") x):rest),
+            [Replace Stmt (toSS g) (("x", toSS x):zip vs (toSS <$> f:fs)) (intercalate " . " (take (length fs + 1) vs) ++ " <$> x"), Delete Stmt (toSS q)])
+  where vs = ('f':) . show <$> [0..]
+        notDol (InfixApp _ _ op _) = not $ isDol op
+        notDol _ = True
+monadFmap _ = Nothing
+
 monadReturn :: [Stmt S] -> Maybe ([Stmt S], [Refactoring R.SrcSpan])
 monadReturn (reverse -> q@(Qualifier _ (App _ ret (Var _ v))):g@(Generator _ (PVar _ p) x):rest)
-    | ret ~= "return", fromNamed v == fromNamed p
+    | isReturn ret, fromNamed v == fromNamed p
     = Just (reverse (Qualifier an x : rest),
             [Replace Stmt (toSS g) [("x", toSS x)] "x", Delete Stmt (toSS q)])
 monadReturn _ = Nothing
@@ -132,5 +153,5 @@ monadLet xs = if null rs then Nothing else Just (ys, rs)
 
 fromRet (Paren _ x) = fromRet x
 fromRet (InfixApp _ x y z) | opExp y ~= "$" = fromRet $ App an x z
-fromRet (App _ x y) | x ~= "return" = Just y
+fromRet (App _ x y) | isReturn x = Just y
 fromRet _ = Nothing
