@@ -9,6 +9,7 @@ module Idea(
     Severity(..)
     ) where
 
+import Data.Functor
 import Data.List.Extra
 import Data.Char
 import Numeric
@@ -17,12 +18,13 @@ import Config.Type
 import HsColour
 import Refact.Types hiding (SrcSpan)
 import qualified Refact.Types as R
+import Prelude
 
 
 -- | An idea suggest by a 'Hint'.
 data Idea = Idea
-    {ideaModule :: String -- ^ The module the idea applies to, may be @\"\"@ if the module cannot be determined or is a result of cross-module hints.
-    ,ideaDecl :: String -- ^ The declaration the idea applies to, typically the function name, but may be a type name.
+    {ideaModule :: [String] -- ^ The modules the idea is for, usually a singleton.
+    ,ideaDecl :: [String] -- ^ The declarations the idea is for, usually a singleton, typically the function name, but may be a type name.
     ,ideaSeverity :: Severity -- ^ The severity of the idea, e.g. 'Warning'.
     ,ideaHint :: String -- ^ The name of the hint that generated the idea, e.g. @\"Use reverse\"@.
     ,ideaSpan :: SrcSpan -- ^ The source code the idea relates to.
@@ -37,9 +39,9 @@ data Idea = Idea
 -- 1) Aeson doesn't esape unicode characters, and I want to (allows me to ignore encoding)
 -- 2) I want to control the format so it's slightly human readable as well
 showIdeaJson :: Idea -> String
-showIdeaJson idea@Idea{ideaSpan=srcSpan@SrcSpan{..}, ..} = wrap . intercalate "," . map mkPair $
-    [("module", str ideaModule)
-    ,("decl", str ideaDecl)
+showIdeaJson idea@Idea{ideaSpan=srcSpan@SrcSpan{..}, ..} = dict
+    [("module", list $ map str ideaModule)
+    ,("decl", list $ map str ideaDecl)
     ,("severity", str $ show ideaSeverity)
     ,("hint", str ideaHint)
     ,("file", str srcSpanFilename)
@@ -49,7 +51,7 @@ showIdeaJson idea@Idea{ideaSpan=srcSpan@SrcSpan{..}, ..} = wrap . intercalate ",
     ,("endColumn", show srcSpanEndColumn)
     ,("from", str ideaFrom)
     ,("to", maybe "null" str ideaTo)
-    ,("note", "[" ++ intercalate "," (map (str . show) ideaNote) ++ "]")
+    ,("note", list (map (str . show) ideaNote))
     ,("refactorings", str $ show ideaRefactoring)
     ]
   where
@@ -60,8 +62,8 @@ showIdeaJson idea@Idea{ideaSpan=srcSpan@SrcSpan{..}, ..} = wrap . intercalate ",
               f '\r' = "\\r"
               f x | isControl x || not (isAscii x) = "\\u" ++ takeEnd 4 ("0000" ++ showHex (ord x) "")
               f x = [x]
-    mkPair (k, v) = show k ++ ":" ++ v
-    wrap x = "{" ++ x ++ "}"
+    dict xs = "{" ++ intercalate "," [show k ++ ":" ++ v | (k,v) <- xs] ++ "}"
+    list xs = "[" ++ intercalate "," xs ++ "]"
 
 showIdeasJson :: [Idea] -> String
 showIdeasJson ideas = "[" ++ intercalate "\n," (map showIdeaJson ideas) ++ "]"
@@ -71,9 +73,7 @@ instance Show Idea where
 
 
 showANSI :: IO (Idea -> String)
-showANSI = do
-    f <- hsColourConsole
-    return $ showEx f
+showANSI = showEx <$> hsColourConsole
 
 showEx :: (String -> String) -> Idea -> String
 showEx tt Idea{..} = unlines $
@@ -87,8 +87,8 @@ showEx tt Idea{..} = unlines $
             where xs = lines $ tt x
 
 
-rawIdea = Idea "" ""
-rawIdeaN a b c d e f = Idea "" "" a b c d e f []
+rawIdea = Idea [] []
+rawIdeaN a b c d e f = Idea [] [] a b c d e f []
 
 idea severity hint from to = rawIdea severity hint (srcInfoSpan $ ann from) (f from) (Just $ f to) []
     where f = trimStart . prettyPrint
