@@ -1,10 +1,12 @@
-{-# LANGUAGE PatternGuards, RecordWildCards #-}
+{-# LANGUAGE PatternGuards, RecordWildCards, ViewPatterns #-}
 
 -- | Check the <TEST> annotations within source and hint files.
 module Test.Annotations(testAnnotations) where
 
+import Control.Exception.Extra
 import Data.Tuple.Extra
 import Data.Char
+import Data.Either.Extra
 import Data.List.Extra
 import Data.Maybe
 import Data.Function
@@ -14,6 +16,8 @@ import Idea
 import Apply
 import HSE.All
 import Test.Util
+import Data.Functor
+import Prelude
 
 
 -- Input, Output
@@ -27,18 +31,20 @@ testAnnotations setting file = do
     mapM_ f tests
     where
         f (Test loc inp out) = do
-            ideas <- applyHintFile defaultParseFlags setting file $ Just inp
-            let good = case out of
-                    Nothing -> null ideas
-                    Just x -> length ideas == 1 &&
-                              seq (length (show ideas)) True && -- force, mainly for hpc
-                              match x (head ideas)
+            ideas <- try_ $ do
+                res <- applyHintFile defaultParseFlags setting file $ Just inp
+                evaluate $ length $ show res
+                return res
+            let good = case (out, ideas) of
+                    (Nothing, Right []) -> True
+                    (Just x, Right [idea]) | match x idea -> True
+                    _ -> False
             let bad =
                     [failed $
-                        ["TEST FAILURE (" ++ show (length ideas) ++ " hints generated)"
+                        ["TEST FAILURE (" ++ show (either (const 1) length ideas) ++ " hints generated)"
                         ,"SRC: " ++ showSrcLoc loc
                         ,"INPUT: " ++ inp] ++
-                        map ((++) "OUTPUT: " . show) ideas ++
+                        map ("OUTPUT: " ++) (either (return . show) (map show) ideas) ++
                         ["WANTED: " ++ fromMaybe "<failure>" out]
                         | not good] ++
                     [failed
@@ -46,23 +52,23 @@ testAnnotations setting file = do
                         ,"SRC: " ++ showSrcLoc loc
                         ,"INPUT: " ++ inp
                         ,"OUTPUT: " ++ show i]
-                        | i@Idea{..} <- ideas, let SrcLoc{..} = getPointLoc ideaSpan, srcFilename == "" || srcLine == 0 || srcColumn == 0]
+                        | i@Idea{..} <- fromRight [] ideas, let SrcLoc{..} = getPointLoc ideaSpan, srcFilename == "" || srcLine == 0 || srcColumn == 0]
             if null bad then passed else sequence_ bad
 
         match "???" _ = True
-        match x y | "@" `isPrefixOf` x = a == show (ideaSeverity y) && match (trimStart b) y
-            where (a,b) = break isSpace $ tail x
-        match x y = on (==) norm (fromMaybe "" $ ideaTo y) x
+        match (word1 -> ("@Note",note)) i = map show (ideaNote i) == [note]
+        match "@NoNote" i = null (ideaNote i)
+        match (word1 -> ('@':sev, msg)) i = sev == show (ideaSeverity i) && match msg i
+        match msg i = on (==) norm (fromMaybe "" $ ideaTo i) msg
 
         -- FIXME: Should use a better check for expected results
         norm = filter $ \x -> not (isSpace x) && x /= ';'
 
 
 parseTestFile :: FilePath -> IO [Test]
-parseTestFile file = do
-    src <- readFile file
+parseTestFile file =
     -- we remove all leading # symbols since Yaml only lets us do comments that way
-    return $ f False $ zip [1..] $ map (\x -> fromMaybe x $ stripPrefix "# " x) $ lines src
+    f False . zip [1..] . map (\x -> fromMaybe x $ stripPrefix "# " x) . lines <$> readFile file
     where
         open = isPrefixOf "<TEST>"
         shut = isPrefixOf "</TEST>"
